// Copyright (c) 2023 Tudor Stanciu

import { getUrlTemplates } from "../config";
import { TuitioClient } from "../client";
import type { TuitioAuthenticationResult, TuitioToken } from "../client";

test("Get url templates", () => {
  const result = getUrlTemplates("https://test.com/api");
  expect(result).toHaveProperty("authentication");
  expect(result.authentication).toContain("https://test.com/api");
  expect(result.authentication).toContain("{username}");
  expect(result.authentication).toContain("{password}");
});

test("Tuitio client initialization", () => {
  const client = new TuitioClient("https://test.com/api");
  expect(client.tuitioUrl).toBe("https://test.com/api");
});

test("Tuitio client initialization with null URL", () => {
  try {
    new TuitioClient(null);
  } catch (error: any) {
    expect(error.message).toContain("is not a valid URL");
  }
});

test("Tuitio client initialization with invalid URL", () => {
  try {
    new TuitioClient("test");
  } catch (error: any) {
    expect(error.message).toContain("is not a valid URL");
  }
});

test("Tuitio client fake authentication", async () => {
  const client = new TuitioClient("https://test.com/api");
  const spy = jest.spyOn(client, "authenticate").mockImplementation(async (userName, password) => {
    const token = <TuitioToken>{ raw: `mock-${userName}-${password}`, validFrom: new Date("10/02/2023"), validUntil: new Date("11/02/2023") };
    const authResult = <TuitioAuthenticationResult>{ token, status: "_MOCK_" };
    return authResult;
  });

  const result = await client.authenticate("user", "pass");

  expect(result.token.raw).toBe("mock-user-pass");
  expect(result.token.validFrom).toStrictEqual(new Date("10/02/2023"));
  expect(result.token.validUntil).toStrictEqual(new Date("11/02/2023"));
  expect(result.status).toBe("_MOCK_");

  spy.mockRestore();
});
